program poordenurk;
type punkt = record
         x, y : real;
       end;
var f : text;
    A, B : punkt;
    vastus : real;

{
Arvutame va"lja ja"rgmise determinandi:
 | p1.x p1.y  1 |
 | p2.x p2.y  1 |
 | p3.x p3.y  1 |
Kes ei tea, mis determinant on, siis vo~ib selle selgituse leida
matemaatikaleksikonist vo~i mo~nest keskkooli matemaatikao~pikust.
Kui see determinant on positiivne, siis ta"hendab see seda, et po"o"re
liikumisel punktist p1 punkti p3 la"bi punkti p2 toimub vastupa"eva,
kui negatiivne, siis pa"ripa"eva ning kui null, siis on punktid p1, p2 ja
p3 u"hel sirgel. Sedasama saab a"ra kasutada ka va"ga paljude teistsuguste
u"lesannete lahendamisel. Meie kasutame a"ra seda, et kui liikumisel
punktist O punkti B la"bi punkti A on po"o"re vastupa"eva, siis on soovitud
nurk alla 180 kraadi ning kui pa"ripa"eva, siis u"le 180 kraadi.

Antud u"lesandesse mitte puutuv jutt :
Osutub, et selle determinandi absoluutva"a"rtus on vo~rdne kolmnurga
p1p2p3 pindalaga. Nii et teisitiso~nastatult vo~iks o"elda, et
see determinant on S, kui p1, p2 ja p3 on kolmnurgas vastupa"eva,
-S, kui p1, p2 ja p3 on kolmnurgas pa"ripa"eva ning 0, kui p1, p2 ja
p3 on u"hel sirgel. See determinant annab kolmnurga p1p2p3 ma"rgiga pindala.
}
function determinant111(p1,p2,p3:punkt):real;
  begin
    determinant111:=p1.x*p2.y + p2.x*p3.y + p3.x*p1.y
                  - p1.y*p2.x - p2.y*p3.x - p3.y*p1.x;
  end;

function poore_vastupaeva(p1,p2,p3:punkt):boolean;
  begin
    poore_vastupaeva := determinant111(p1,p2,p3) >= 0;
  end;

{
Ta"histame a"ra kolmnurga AOB ku"ljed :
a = OA, b = OB, c = AB.
Nurga leidmiseks kasutame koosinusteoreemi :
sqr(c) = sqr(a) + sqr(b) - 2*a*b*cos(nurk)
kus 'nurk' on nurk ku"lgede a ja b vahel.
Siit saame, et
cos(nurk) = (sqr(a) + sqr(b) - sqr(c)) / (2*a*b)
nurk = arccos( (sqr(a) + sqr(b) - sqr(c)) / (2*a*b) )
Arkuskoosinusega arvust x saame sellise nurga 0..Pi, mille koosinus on x.
Seepa"rast tuleb juhul, kui po"o"re oli pa"ripa"eva, lahutada arvust 2*Pi
see leitud nurk (vt. funktsioon nurk).

Valemis on kasutatud ku"lgede AO, BO ja AB pikkusi. Nende leidmiseks kasutame
valemit kahe punkti kauguse kohta (vt. funktsioon kaugus).

Veel on kasutatud arkuskoosinust. Pascali standardvahendites arkuskoosinust
ei ole, on vaid arkustangens. Arkustangensi (ArcTan) helpi lugedes leiab
sealt valemi arkuskoosinuse arvutamiseks arkustangensi kaudu. Kahjuks
kehtib see vaid nende arvude korral, mis ja"a"vad 0 ja 1 vahele. Meil on
lisaks vaja ka selle va"a"rtust negatiivsete argumentide korral. Koosinuse
graafiku ja"rgi on na"ha, et kui x<0, siis arccos(x) = Pi - arccos(-x).
Punktides 0 ja 1 kirjutame va"a"rtused eraldi va"lja (tegelikult tuleb
va"a"rtus punktis 1 ka arkustangensi kaudu va"lja, kuid olu"mpiaadil pole
alati aega selliste asjade vaatamiseks - lihtsam on tingimus eraldi va"lja
kirjutada).
}
function arccos(x:real):real;
  begin
    if x<0 then
      arccos := Pi - arccos(-x)
    else if x=0 then
      arccos := Pi/2
    else if (x>0) and (x<1) then
      arccos := arctan ( sqrt(1-sqr(x)) / x )
    else if x=1 then
      arccos := 0
    else
      runerror(207); {invalid floating point operation}
       { Kui anda funktsioonile arv, mis on suurem kui 1 vo~i va"iksem
         kui -1, siis katkestab programm to"o" veateatega (arkuskoosinus
         pole sellisel juhul ma"a"ratud) }
  end;

function kaugus(P1,P2:punkt):real;
  begin
    kaugus := sqrt( sqr(P1.x-P2.x) + sqr(P1.y-P2.y) );
      { see valem tuleb Pythagorase teoreemist }
  end;

function nurk(A,B:punkt):real; { arvutab u"lesandes ku"situd }
var O : punkt;                 { nurga radiaanides }
    x, y, z : real;
  begin
    O.x:=0;  O.y:=0;
    if poore_vastupaeva(O,A,B) then
      begin
        x:=kaugus(O,A);
        y:=kaugus(O,B);
        z:=kaugus(A,B);
        nurk := arccos( (sqr(x)+sqr(y)-sqr(z)) / (2*x*y) );
      end
    else
      nurk := 2*Pi - nurk(B,A);
  end;

{
Nu"u"d on ja"a"nud veel teisendada vastus kraadidest radiaanideks.
Selleks on vaja teada, et ta"isring on 2*Pi radiaani ehk 360 kraadi.
x radiaani = x/(2*Pi) ta"isringi = x/(2*Pi)*360 kraadi.
}
function radiaanid_kraadideks(x:real):real;
  begin
    radiaanid_kraadideks := (x/(2*Pi)) * 360;
  end;

begin
  assign(f,'NURK.SIS');
  reset(f);
  readln(f,A.x,A.y);
  readln(f,B.x,B.y);
  close(f);

  vastus := radiaanid_kraadideks( nurk(A,B) );

  assign(f,'NURK.VAL');
  rewrite(f);
  writeln(f,vastus:0:10);
  close(f);
end.
